@@ -70,7 +70,7 @@ struct s2n_config {
     unsigned disable_x509_validation : 1;
     unsigned max_verify_cert_chain_depth_set : 1;
     /* Whether to add dss cert type during a server certificate request.
-     * See https://github.com/aws/s2n-tls/blob/main/docs/USAGE-GUIDE.md */
+     * See s2n_config_enable_cert_req_dss_legacy_compat. */
     unsigned cert_req_dss_legacy_compat_enabled : 1;
     /* Whether any RSA certificates have been configured server-side to send to clients. This is needed so that the
      * server knows whether or not to self-downgrade to TLS 1.2 if the server is compiled with Openssl 1.0.2 and does
@@ -91,7 +91,7 @@ struct s2n_config {
 
     /* Indicates s2n_recv should read as much as it can into the output buffer
      *
-     * Note: This defaults to false to ensure backwards compatability with
+     * Note: This defaults to false to ensure backwards compatibility with
      * applications which relied on s2n_recv returning a single record.
      */
     unsigned recv_multi_record : 1;
@@ -102,6 +102,9 @@ struct s2n_config {
     /* Indicates whether s2n has enabled OCSP status requests, for backwards compatibility */
     unsigned ocsp_status_requested_by_s2n : 1;
 
+    /* TLS1.3 can be dangerous with kTLS. Require it to be explicitly enabled. */
+    unsigned ktls_tls13_enabled : 1;
+
     struct s2n_dh_params *dhparams;
     /* Needed until we can deprecate s2n_config_add_cert_chain_and_key. This is
      * used to release memory allocated only in the deprecated API that the application 
@@ -200,6 +203,7 @@ struct s2n_config {
 S2N_CLEANUP_RESULT s2n_config_ptr_free(struct s2n_config **config);
 
 int s2n_config_defaults_init(void);
+S2N_RESULT s2n_config_testing_defaults_init_tls13_certs(void);
 struct s2n_config *s2n_fetch_default_config(void);
 int s2n_config_set_unsafe_for_testing(struct s2n_config *config);
 
@@ -208,5 +212,10 @@ int s2n_config_free_session_ticket_keys(struct s2n_config *config);
 
 void s2n_wipe_static_configs(void);
 struct s2n_cert_chain_and_key *s2n_config_get_single_default_cert(struct s2n_config *config);
-int s2n_config_get_num_default_certs(struct s2n_config *config);
+int s2n_config_get_num_default_certs(const struct s2n_config *config);
 S2N_RESULT s2n_config_wall_clock(struct s2n_config *config, uint64_t *output);
+
+/* Validate that the certificates in `config` respect the certificate preferences
+ * in `security_policy` */
+S2N_RESULT s2n_config_validate_loaded_certificates(const struct s2n_config *config,
+        const struct s2n_security_policy *security_policy);
